import _extends from "@babel/runtime/helpers/extends";
import remark from 'remark';
import { getTransformValue } from './transform';
import { FUNNAME_PREFIX } from '../';

/**
 * Creates an object containing the parameters of the current URL.
 *
 * ```js
 * getURLParameters('name=Adam&surname=Smith');
 * // 👉 {name: 'Adam', surname: 'Smith'}
 * ```
 * @param url `name=Adam&surname=Smith`
 * @returns
 */
export var getURLParameters = url => {
  var regex = /([^?=&]+)=([^&]*)/g;
  var params = {};
  var match;
  while (match = regex.exec(url)) {
    params[match[1]] = match[2];
  }
  return params;
};
/** 转换 代码*/
export var getProcessor = source => {
  try {
    // @ts-ignore
    var child = remark.parse(source);
    return child.children;
  } catch (err) {
    console.warn(err);
  }
};

/**
 * ```js
 * 'mdx:preview' => ''  // Empty
 * 'mdx:preview:demo12' => 'demo12' // return meta id => 'demo12'
 * ```
 * @param meta string
 * @returns string?
 */
export var getMetaId = function getMetaId(meta) {
  if (meta === void 0) {
    meta = '';
  }
  var [metaRaw = ''] = /mdx:(.[\w|:]+)/i.exec(meta) || [];
  return metaRaw.replace(/^mdx:preview:?/, '');
};

/**
 * ```js
 * isMeta('mdx:preview') => true
 * isMeta('mdx:preview:demo12') => true
 * isMeta('mdx:preview--demo12') => false
 * ```
 * @param meta
 * @returns boolean
 */
export var isMeta = function isMeta(meta) {
  if (meta === void 0) {
    meta = '';
  }
  return meta && meta.includes('mdx:preview');
};

/** 获取需要渲染的代码块 **/
export function getCodeBlock(child, opts, resourcePath) {
  if (opts === void 0) {
    opts = {};
  }
  var {
    lang = ['jsx', 'tsx']
  } = opts;
  // 获取渲染部分
  var codeBlock = {};
  child.forEach(item => {
    if (item && item.type === 'code' && lang.includes(item.lang)) {
      var line = item.position.start.line;
      var metaId = getMetaId(item.meta);
      if (isMeta(item.meta)) {
        var name = metaId || line;
        var funName = resourcePath + "." + FUNNAME_PREFIX + name;
        var returnCode = getTransformValue(item.value, funName + "." + item.lang, opts);
        codeBlock[name] = {
          name,
          meta: getURLParameters(item.meta),
          code: returnCode,
          language: item.lang,
          value: item.value
        };
      }
    }
  });
  return codeBlock;
}

/**
 * `mdCodeModulesLoader` method for adding `markdown-react-code-preview-loader` to webpack config.
 * @param {webpack.Configuration} config webpack config
 * @param {string[]} lang Parsing language
 * @param {Options} option Loader Options
 * @returns {webpack.Configuration}
 */
export var mdCodeModulesLoader = function mdCodeModulesLoader(config, lang, option) {
  if (option === void 0) {
    option = {};
  }
  config.module.rules.forEach(ruleItem => {
    if (typeof ruleItem === 'object') {
      if (ruleItem.oneOf) {
        ruleItem.oneOf.unshift({
          test: /.md$/,
          use: [{
            loader: 'markdown-react-code-preview-loader',
            options: _extends({
              lang
            }, option)
          }]
        });
      }
    }
  });
  return config;
};
/**进行获取同级别标题数据*/
export var getSameLevelHeading = list => {
  var newList = [];
  var level = 0;
  var satrtIndex = 0;
  var lg = list.length;

  // 对同级别数据进行区分
  for (var index = 0; index < lg; index++) {
    var element = list[index];
    if (index === 0) {
      satrtIndex = 0;
      /**默认第一个数据的层级进行查找*/
      level = element.depth;
    } else if (element.depth === level) {
      // 层级相同则进行赋值
      // 这个位置相等，说明这些数据是一组数据
      newList.push({
        start: satrtIndex,
        end: index
      });
      /**重新赋值开始下标数据*/
      satrtIndex = index;
    }
  }
  // 如果最后位置没找到
  if (satrtIndex <= lg - 1) {
    newList.push({
      start: satrtIndex,
      end: lg
    });
  }
  var saveList = [];

  /**对标题数据进行处理*/
  newList.forEach(item => {
    var {
      start,
      end
    } = item;
    var [firstItem, ...lastItems] = list.slice(start, end);
    var newItem = _extends({}, firstItem);
    if (Array.isArray(lastItems) && lastItems.length) {
      newItem.children = getSameLevelHeading(lastItems);
    }
    saveList.push(newItem);
  });
  return saveList;
};

/**获取标题*/
export var getHeadings = child => {
  var headingList = [];
  child.forEach((item, index) => {
    if (item && item.type === 'heading') {
      var {
        depth,
        children
      } = item;
      if (Array.isArray(children) && children.length) {
        var value = children.map(item => item.value).join('');
        headingList.push({
          key: index,
          value,
          depth
        });
      }
    }
  });
  return {
    headings: getSameLevelHeading(headingList),
    headingsList: headingList
  };
};